import { ExtendedCharacter, TSocket } from '../utilities/types'
import prisma from '../utilities/prisma'
import ZoneRepository from '../repositories/zoneRepository'
import ZoneManager from '../managers/zoneManager'
import { Character, ZoneEventTile, ZoneEventTileTeleport } from '@prisma/client'
import { Server } from 'socket.io'

export class ZoneEventTileService {
  public async handleTeleport(io: Server, socket: TSocket, character: ExtendedCharacter, teleport: ZoneEventTileTeleport): Promise<void> {
    if (teleport.toZoneId === character.zoneId) return

    const zone = await ZoneRepository.getById(teleport.toZoneId)
    if (!zone) return

    const oldZoneId = character.zoneId
    const newZoneId = teleport.toZoneId

    character.zoneId = teleport.toZoneId
    character.positionX = teleport.toPositionX
    character.positionY = teleport.toPositionY

    socket.leave(character.zoneId.toString())
    socket.join(teleport.toZoneId.toString())

    io.to(oldZoneId.toString()).emit('zone:character:leave', character);
    io.to(newZoneId.toString()).emit('zone:character:join', character);
    ZoneManager.removeCharacterFromZone(oldZoneId, character as Character);
    ZoneManager.addCharacterToZone(newZoneId, character as Character);

    socket.emit('zone:teleport', {
      zone,
      characters: ZoneManager.getCharactersInZone(zone.id)
    })
  }
}
